@@ -1,46 +1,38 @@
-from __future__ import annotations
+import functools
+from pydantic import BaseModel, FilePath, DirectoryPath, validator
+from typing import Any
+from typing import List, Optional
 
 import pandas as pd
-import functools
-import os
-from dataclasses import dataclass
-from typing import Any, Union
-from pathlib import Path
-from typing import List, Optional, Union, Any
-from pydantic import BaseModel, dataclasses, fields, Field
-import yaml
-import json
-import os
-import dataclasses
-from typing import Union, Dict, Any, TypedDict
-from pathlib import Path
-
 from aws_pcluster_helpers import (
     PClusterConfig,
-    InstanceTypesData,
     PClusterInstanceTypes,
     InstanceTypesMappings,
     size_in_gib,
 )
-from aws_pcluster_helpers.utils.logging import setup_logger
 from aws_pcluster_helpers.models.config import PClusterConfigFiles
-from rich.console import Console
+from aws_pcluster_helpers.utils.logging import setup_logger
+from pydantic import BaseModel
+from pcluster.config.cluster_config import SlurmClusterConfig
+from pcluster.schemas.cluster_schema import ClusterSchema
 from rich.table import Table
 
 logger = setup_logger(logger_name="sinfo")
 
 
-@dataclasses.dataclass
-class SinfoRow:
-    sinfo_name: str
-    label: str
-    queue: str
-    constraint: str
-    ec2_instance_type: str
-    mem: int
-    cpu: int
+class SinfoRow(BaseModel):
+    sinfo_name: Optional[str]
+    label: Optional[str]
+    queue: Optional[str]
+    constraint: Optional[str]
+    ec2_instance_type: Optional[str]
+    mem: Optional[int]
+    mem_mib: Optional[int]
+    cpu: Optional[int]
+    scheduleable_memory: Optional[int] = 0.95
+    scheduleable_memory_mib: Optional[int]
+    scheduleable_memory_gib: Optional[int]
     vcpu: Optional[int]
-    # gpu: Optional[dict]
     gpus: Optional[List]
     extra: Optional[dict]
 
@@ -48,51 +40,58 @@ class SinfoRow:
 # TODO add custom ami lookup
 
 
-@dataclass
-class SInfoTable:
-    @functools.cached_property
-    def table_columns(self) -> List[Any]:
-        return [
-            {"label": "Queue", "key": "queue"},
-            {"label": "Constraint", "key": "constraint"},
-            {"label": "Mem(Gib)", "key": "mem"},
-            {"label": "VCPU", "key": "vcpu"},
-            {"label": "CPU", "key": "cpu"},
-            {"label": "EC2", "key": "ec2_instance_type"},
-        ]
+class SInfoTable(BaseModel):
+    class Config:
+        arbitrary_types_allowed = True
 
-    @functools.cached_property
-    def pcluster_config_files(self) -> PClusterConfigFiles:
-        return PClusterConfigFiles()
+    table_columns: List[Any] = [
+        {"label": "Queue", "key": "queue"},
+        {"label": "Constraint", "key": "constraint"},
+        {"label": "Mem(Gib)", "key": "mem"},
+        {"label": "VCPU", "key": "vcpu"},
+        {"label": "CPU", "key": "cpu"},
+        {"label": "EC2", "key": "ec2_instance_type"},
+    ]
+    pcluster_config_files: PClusterConfigFiles = PClusterConfigFiles()
+    instance_type_mappings: Optional[InstanceTypesMappings]
+    pcluster_instance_types: Optional[PClusterInstanceTypes]
+    pcluster_config: Optional[SlurmClusterConfig]
+    rows: Optional[List[SinfoRow]]
+    dataframe: Optional[pd.DataFrame]
 
-    @functools.cached_property
-    def instance_type_mappings(self) -> InstanceTypesMappings:
+    @validator("instance_type_mappings", pre=True, always=True)
+    def set_instance_type_mappings(cls, v, values, **kwargs):
+        pcluster_config_files = values.get("pcluster_config_files")
         return InstanceTypesMappings.from_json(
-            self.pcluster_config_files.instance_name_type_mappings_file
+            pcluster_config_files.instance_name_type_mappings_file
         )
 
-    @functools.cached_property
-    def pcluster_instance_types(self) -> PClusterInstanceTypes:
+    @validator("pcluster_instance_types", pre=True, always=True)
+    def set_pcluster_instance_types(cls, v, values, **kwargs):
+        pcluster_config_files = values.get("pcluster_config_files")
         return PClusterInstanceTypes.from_json(
-            self.pcluster_config_files.instance_types_data_file
+            pcluster_config_files.instance_types_data_file
         )
 
-    @functools.cached_property
-    def pcluster_config(self) -> PClusterConfig:
-        return PClusterConfig.from_yaml(self.pcluster_config_files.pcluster_config_file)
+    @validator("pcluster_config", pre=True, always=True)
+    def set_pcluster_config(cls, v, values, **kwargs):
+        pcluster_config_files = values.get("pcluster_config_files")
+        return PClusterConfig.from_yaml(pcluster_config_files.pcluster_config_file)
 
-    @functools.cached_property
-    def rows(self) -> List[SinfoRow]:
-        instance_types_mappings = self.instance_type_mappings
-        pcluster_instance_types = self.pcluster_instance_types
+    @validator("rows", pre=True, always=True)
+    def set_rows(cls, v, values, **kwargs) -> List[SinfoRow]:
+        pcluster_config_files = values.get("pcluster_config_files")
+        instance_types_mappings = values.get("instance_type_mappings")
+        pcluster_instance_types = values.get("pcluster_instance_types")
+        pcluster_config = values.get("pcluster_config")
 
         sinfo_records = []
-        for slurm_queue in self.pcluster_config.Scheduling.SlurmQueues:
-            compute_resources = slurm_queue.ComputeResources
-            queue = slurm_queue.Name
+        for slurm_queue in pcluster_config.scheduling.queues:
+            compute_resources = slurm_queue.compute_resources
+            queue = slurm_queue.name
             for compute_resource in compute_resources:
                 sinfo_label = {}
-                sinfo_instance_type = compute_resource.Name
+                sinfo_instance_type = compute_resource.name
                 ec2_instance_type = instance_types_mappings.sinfo_instance_types[
                     sinfo_instance_type
                 ]["ec2_instance_type"]
@@ -113,10 +112,10 @@ class SInfoTable:
                 cpu = pcluster_instance_type.data["data"]["VCpuInfo"]["DefaultCores"]
                 vcpu = pcluster_instance_type.data["data"]["VCpuInfo"]["DefaultVCpus"]
                 p_type = "dy"
-                if compute_resource.MinCount > 0:
+                if compute_resource.min_count > 0:
                     p_type = "st"
-                label = f"{slurm_queue.Name.replace('-', '_')}_{p_type}__{ec2_instance_type.replace('.', '_')}"
-                sinfo_name = f"{slurm_queue.Name}-{p_type}-{sinfo_instance_type}"
+                label = f"{slurm_queue.name.replace('-', '_')}_{p_type}__{ec2_instance_type.replace('.', '_')}"
+                sinfo_name = f"{slurm_queue.name}-{p_type}-{sinfo_instance_type}"
 
                 sinfo_label["name"] = label
                 sinfo_label["max_mem"] = int(mem_in_gib)
@@ -124,10 +123,20 @@ class SInfoTable:
                 sinfo_label["queue"] = queue
                 sinfo_label["constraint"] = sinfo_instance_type
                 sinfo_label["vcpu"] = vcpu
+                scheduleable_memory = getattr(
+                    compute_resource, "scheduleable_memory", 0.95
+                )
+                scheduleable_memory_mib = mem_in_mib * scheduleable_memory
+                scheduleable_memory_gib = mem_in_gib * scheduleable_memory
+
                 sinfo_records.append(
                     SinfoRow(
                         mem=int(mem_in_gib),
+                        mem_mib=int(mem_in_mib),
                         cpu=cpu,
+                        scheduleable_memory=scheduleable_memory,
+                        scheduleable_memory_mib=scheduleable_memory_mib,
+                        scheduleable_memory_gib=scheduleable_memory_gib,
                         vcpu=vcpu,
                         sinfo_name=sinfo_name,
                         queue=queue,
@@ -141,10 +150,11 @@ class SInfoTable:
                 )
         return sinfo_records
 
-    @functools.cached_property
-    def dataframe(self) -> pd.DataFrame:
+    @validator("dataframe", pre=True, always=True)
+    def set_dataframe(cls, v, values, **kwargs) -> pd.DataFrame:
         records = []
-        for record in self.rows:
+        rows = values.get("rows")
+        for record in rows:
             records.append(record.__dict__)
         df = pd.DataFrame.from_records(records)
         df.sort_values(by=["queue", "mem", "vcpu", "cpu"], inplace=True)
@@ -184,7 +194,7 @@ class SInfoTable:
                 value = row[key]
                 t_row.append(f"[{color}]{str(value)}")
             t_row.append(
-                f"[{color}]#SBATCH --partition={queue}\n#SBATCH --constraint={constraint}\n#SBATCH -c={vcpus}"
+                f"[{color}]#SBATCH --exclusive --partition={queue}\n#SBATCH --constraint={constraint}\n#SBATCH -c={vcpus}"
             )
             table.add_row(*t_row)
             colors_index = colors_index + 1
